# -*- coding: utf-8 -*-


from wechatpy.client.api.base import BaseWeChatAPI


class WeChatKF(BaseWeChatAPI):
    """
    微信客服接口

    https://work.weixin.qq.com/api/doc/90000/90135/94670
    """

    def sync_msg(self, token, cursor="", limit=1000):
        """
        微信客户发送的消息、接待人员在企业微信回复的消息、发送消息接口发送失败事件（如被用户拒收）
        、客户点击菜单消息的回复消息，可以通过该接口获取具体的消息内容和事件。不支持读取通过发送消息接口发送的消息。
        支持的消息类型：文本、图片、语音、视频、文件、位置、链接、名片、小程序、事件。


        :param token: 回调事件返回的token字段，10分钟内有效；可不填，如果不填接口有严格的频率限制。不多于128字节
        :param cursor: 上一次调用时返回的next_cursor，第一次拉取可以不填。不多于64字节
        :param limit: 期望请求的数据量，默认值和最大值都为1000。
        注意：可能会出现返回条数少于limit的情况，需结合返回的has_more字段判断是否继续请求。
        :return: 接口调用结果
        """
        data = {
            "token": token,
            "cursor": cursor,
            "limit": limit,
        }
        return self._post("kf/sync_msg", data=data)

    def get_service_state(self, open_kfid, external_userid):
        """
        获取会话状态

        ID	状态	说明
        0	未处理	新会话接入。可选择：1.直接用API自动回复消息。2.放进待接入池等待接待人员接待。3.指定接待人员进行接待
        1	由智能助手接待	可使用API回复消息。可选择转入待接入池或者指定接待人员处理。
        2	待接入池排队中	在待接入池中排队等待接待人员接入。可选择转为指定人员接待
        3	由人工接待	人工接待中。可选择结束会话
        4	已结束	会话已经结束。不允许变更会话状态，等待用户重新发起咨询

        :param open_kfid: 客服帐号ID
        :param external_userid: 微信客户的external_userid
        :return: 接口调用结果
        """
        data = {
            "open_kfid": open_kfid,
            "external_userid": external_userid,
        }
        return self._post("kf/service_state/get", data=data)

    def trans_service_state(self, open_kfid, external_userid, service_state, servicer_userid=""):
        """
        变更会话状态

        :param open_kfid: 客服帐号ID
        :param external_userid: 微信客户的external_userid
        :param service_state: 当前的会话状态，状态定义参考概述中的表格
        :return: 接口调用结果
        """
        data = {
            "open_kfid": open_kfid,
            "external_userid": external_userid,
            "service_state": service_state,
        }
        if servicer_userid:
            data["servicer_userid"] = servicer_userid
        return self._post("kf/service_state/trans", data=data)

    def get_servicer_list(self, open_kfid):
        """
        获取接待人员列表

        :param open_kfid: 客服帐号ID
        :return: 接口调用结果
        """
        data = {
            "open_kfid": open_kfid,
        }
        return self._get("kf/servicer/list", params=data)

    def add_servicer(self, open_kfid, userid_list):
        """
        添加接待人员
        添加指定客服帐号的接待人员。

        :param open_kfid: 客服帐号ID
        :param userid_list: 接待人员userid列表
        :return: 接口调用结果
        """
        if not isinstance(userid_list, list):
            userid_list = [userid_list]

        data = {
            "open_kfid": open_kfid,
            "userid_list": userid_list,
        }
        return self._post("kf/servicer/add", data=data)

    def del_servicer(self, open_kfid, userid_list):
        """
        删除接待人员
        从客服帐号删除接待人员

        :param open_kfid: 客服帐号ID
        :param userid_list: 接待人员userid列表
        :return: 接口调用结果
        """
        if not isinstance(userid_list, list):
            userid_list = [userid_list]

        data = {
            "open_kfid": open_kfid,
            "userid_list": userid_list,
        }
        return self._post("kf/servicer/del", data=data)

    def batchget_customer(self, external_userid_list):
        """
        客户基本信息获取

        :param external_userid_list: external_userid列表
        :return: 接口调用结果
        """
        if not isinstance(external_userid_list, list):
            external_userid_list = [external_userid_list]

        data = {
            "external_userid_list": external_userid_list,
        }
        return self._post("kf/customer/batchget", data=data)

    def get_account_list(self):
        """
        获取客服帐号列表

        :return: 接口调用结果
        """
        return self._get("kf/account/list")

    def add_contact_way(self, open_kfid, scene):
        """
        获取客服帐号链接

        :param open_kfid: 	客服帐号ID
        :param scene: 场景值，字符串类型，由开发者自定义。不多于32字节;字符串取值范围(正则表达式)：[0-9a-zA-Z_-]*
        :return: 接口调用结果
        """
        data = {"open_kfid": open_kfid, "scene": scene}
        return self._post("kf/add_contact_way", data=data)

    def get_upgrade_service_config(self):
        """
        获取配置的专员与客户群

        :return: 接口调用结果
        """
        return self._get("kf/customer/get_upgrade_service_config")

    def upgrade_service(self, open_kfid, external_userid, service_type, member=None, groupchat=None):
        """
        为客户升级为专员或客户群服务

        :param open_kfid: 	客服帐号ID
        :param external_userid: 微信客户的external_userid
        :param service_type: 表示是升级到专员服务还是客户群服务。1:专员服务。2:客户群服务
        :param member: 推荐的服务专员，type等于1时有效
        :param groupchat: 推荐的客户群，type等于2时有效
        :return: 接口调用结果
        """

        data = {
            "open_kfid": open_kfid,
            "external_userid": external_userid,
            "type": service_type,
        }
        if service_type == 1:
            data["member"] = member
        else:
            data["groupchat"] = groupchat
        return self._post("kf/customer/upgrade_service", data=data)

    def cancel_upgrade_service(self, open_kfid, external_userid):
        """
        为客户取消推荐

        :param open_kfid: 	客服帐号ID
        :param external_userid: 微信客户的external_userid
        :return: 接口调用结果
        """

        data = {"open_kfid": open_kfid, "external_userid": external_userid}
        return self._post("kf/customer/cancel_upgrade_service", data=data)

    def send_msg_on_event(self, code, msgtype, msg_content, msgid=None):
        """
        当特定的事件回调消息包含code字段，可以此code为凭证，调用该接口给用户发送相应事件场景下的消息，如客服欢迎语。
        支持发送消息类型：文本、菜单消息。

        :param code: 事件响应消息对应的code。通过事件回调下发，仅可使用一次。
        :param msgtype: 消息类型。对不同的msgtype，有相应的结构描述，详见消息类型
        :param msg_content: 目前支持文本与菜单消息，具体查看文档
        :param msgid: 消息ID。如果请求参数指定了msgid，则原样返回，否则系统自动生成并返回。不多于32字节；
                      字符串取值范围(正则表达式)：[0-9a-zA-Z_-]*
        :return: 接口调用结果
        """

        data = {"code": code, "msgtype": msgtype}
        if msgid:
            data["msgid"] = msgid
        data.update(msg_content)
        return self._post("kf/send_msg_on_event", data=data)



# -*- coding: utf-8 -*-
from operator import itemgetter

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatMisc(BaseWeChatAPI):
    def short_url(self, long_url):
        """
        将一条长链接转成短链接

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Account_Management/URL_Shortener.html

        该接口即将废弃，详情见公告：
        https://mp.weixin.qq.com/cgi-bin/announce?action=getannouncement&announce_id=11615366683l3hgk&version=63010043&lang=zh_CN&token=

        :param long_url: 长链接地址
        :return: 返回的 JSON 数据包

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    res = client.misc.short_url('http://www.qq.com')

        """
        return self._post("shorturl", data={"action": "long2short", "long_url": long_url})

    def get_wechat_ips(self):
        """
        获取微信服务器 IP 地址列表

        详情请参考：
        https://developers.weixin.qq.com/doc/offiaccount/Basic_Information/Get_the_WeChat_server_IP_address.html

        :return: IP 地址列表

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    ips = client.misc.get_wechat_ips()

        """
        res = self._get("getcallbackip", result_processor=itemgetter("ip_list"))
        return res

    def check_network(self, action="all", operator="DEFAULT"):
        """
        网络检测

        详情请参阅
        https://developers.weixin.qq.com/doc/offiaccount/Basic_Information/Network_Detection.html

        :param action: 执行的检测动作，允许的值：dns（做域名解析）、ping（做ping检测）、all（dns和ping都做）
        :param operator: 指定平台从某个运营商进行检测，允许的值：CHINANET（电信出口）、UNICOM（联通出口）、CAP（腾讯自建出口）、DEFAULT（根据ip来选择运营商）
        :return: 返回的 JSON 数据包
        """
        return self._post("callback/check", data={"action": action, "check_operator": operator})



# -*- coding: utf-8 -*-


from wechatpy.pay.api.base import BaseWeChatPayAPI


class WeChatRefund(BaseWeChatPayAPI):
    def apply(
        self,
        total_fee,
        refund_fee,
        out_refund_no,
        transaction_id=None,
        out_trade_no=None,
        fee_type="CNY",
        op_user_id=None,
        device_info=None,
        refund_account="REFUND_SOURCE_UNSETTLED_FUNDS",
        refund_desc=None,
        notify_url=None,
    ):
        """
        申请退款

        :param total_fee: 订单总金额，单位为分
        :param refund_fee: 退款总金额，单位为分
        :param out_refund_no: 商户系统内部的退款单号，商户系统内部唯一，同一退款单号多次请求只退一笔
        :param transaction_id: 可选，微信订单号
        :param out_trade_no: 可选，商户系统内部的订单号，与 transaction_id 二选一
        :param fee_type: 可选，货币类型，符合ISO 4217标准的三位字母代码，默认人民币：CNY
        :param op_user_id: 可选，操作员帐号, 默认为商户号
        :param device_info: 可选，终端设备号
        :param refund_desc: 可选，退款原因
        :param refund_account: 可选，退款资金来源，仅针对老资金流商户使用，默认使用未结算资金退款
        :param notify_url: 可选，异步接收微信支付退款结果通知的回调地址
        :return: 返回的结果数据
        """
        data = {
            "appid": self.appid,
            "device_info": device_info,
            "transaction_id": transaction_id,
            "out_trade_no": out_trade_no,
            "out_refund_no": out_refund_no,
            "total_fee": total_fee,
            "refund_fee": refund_fee,
            "refund_fee_type": fee_type,
            "op_user_id": op_user_id if op_user_id else self.mch_id,
            "refund_account": refund_account,
            "refund_desc": refund_desc,
            "notify_url": notify_url,
        }
        return self._post("secapi/pay/refund", data=data)

    def query(
        self,
        refund_id=None,
        out_refund_no=None,
        transaction_id=None,
        out_trade_no=None,
        device_info=None,
    ):
        """
        查询退款

        :param refund_id: 微信退款单号
        :param out_refund_no: 商户退款单号
        :param transaction_id: 微信订单号
        :param out_trade_no: 商户系统内部的订单号
        :param device_info: 可选，终端设备号
        :return: 返回的结果数据
        """
        data = {
            "appid": self.appid,
            "device_info": device_info,
            "transaction_id": transaction_id,
            "out_trade_no": out_trade_no,
            "out_refund_no": out_refund_no,
            "refund_id": refund_id,
        }
        return self._post("pay/refundquery", data=data)
